'use strict';

var React = require('react');

var _require = require('./diff'),
    TYPES = _require.TYPES;

var diffNodes = require('./diffNodes');
var NodeWrapper = require('./NodeWrapper');
var Node = require('./Node');

var ChangeWrapper = React.createClass({
    displayName: 'ChangeWrapper',

    propTypes: {
        kind: React.PropTypes.string.isRequired,
        type: React.PropTypes.string.isRequired,
        children: React.PropTypes.node
    },

    render: function render() {
        var _props = this.props,
            kind = _props.kind,
            type = _props.type,
            children = _props.children;

        var className = 'diff-' + kind + '-' + type;

        if (kind == 'block') {
            return React.createElement(
                'div',
                { className: className },
                children
            );
        } else {
            return React.createElement(
                'span',
                { className: className },
                children
            );
        }
    }
});

/**
 * @type {React}
 */
var ChangeNode = React.createClass({
    displayName: 'ChangeNode',

    propTypes: {
        change: React.PropTypes.object.isRequired
    },

    render: function render() {
        var change = this.props.change;
        var type = change.type,
            original = change.original,
            modified = change.modified;


        if (type == TYPES.IDENTICAL) {
            return React.createElement(Node, { node: original });
        } else if (type == TYPES.MODIFIED) {
            if (original.kind == 'text') {
                return React.createElement(ChangeNodes, {
                    Wrapper: function Wrapper(props) {
                        return React.createElement(
                            'span',
                            null,
                            props.children
                        );
                    },
                    original: original.characters,
                    modified: modified.characters
                });
            } else {
                return React.createElement(ChangeNodes, {
                    Wrapper: function Wrapper(props) {
                        return React.createElement(
                            NodeWrapper,
                            { node: original },
                            props.children
                        );
                    },
                    original: original.nodes,
                    modified: modified.nodes
                });
            }
        } else {
            var base = type == TYPES.ADDED ? modified : original;

            return React.createElement(
                ChangeWrapper,
                { type: type, kind: base.kind },
                React.createElement(Node, { node: base })
            );
        }
    }
});

/**
 * @type {React}
 */
var ChangeNodes = React.createClass({
    displayName: 'ChangeNodes',

    propTypes: {
        original: React.PropTypes.object.isRequired,
        modified: React.PropTypes.object.isRequired,
        Wrapper: React.PropTypes.func
    },

    render: function render() {
        var _props2 = this.props,
            Wrapper = _props2.Wrapper,
            original = _props2.original,
            modified = _props2.modified;

        var changes = diffNodes(original, modified);

        var children = changes.map(function (c) {
            return React.createElement(ChangeNode, { key: c.key, change: c });
        });

        return React.createElement(
            Wrapper,
            null,
            children
        );
    }
});

module.exports = ChangeNodes;