'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _toArray(arr) { return Array.isArray(arr) ? arr : Array.from(arr); }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var _require = require('immutable'),
    List = _require.List,
    Record = _require.Record;

var KEY = 0;
var TYPES = {
    IDENTICAL: 'identical',
    MODIFIED: 'modified',
    REMOVED: 'removed',
    ADDED: 'added'
};

var DEFAULTS = {
    type: String(TYPES.IDENTICAL),
    original: null,
    modified: null
};

var Change = function (_Record) {
    _inherits(Change, _Record);

    function Change() {
        _classCallCheck(this, Change);

        return _possibleConstructorReturn(this, (Change.__proto__ || Object.getPrototypeOf(Change)).apply(this, arguments));
    }

    _createClass(Change, null, [{
        key: 'create',
        value: function create(props) {
            return new Change(_extends({
                key: KEY++
            }, props));
        }
    }]);

    return Change;
}(Record(DEFAULTS));

/**
 * Longest common subsequence
 *
 * @param  {Array} xs, ys
 * @param  {Array} head
 * @return {Array}
 */


function lcs(xs, ys, isEqual) {
    if (xs.length > 0 && ys.length > 0) {
        var _xs = _toArray(xs),
            xe = _xs[0],
            xb = _xs.slice(1);

        var _ys = _toArray(ys),
            ye = _ys[0],
            yb = _ys.slice(1);

        if (isEqual(xe, ye)) {
            return [{
                original: xe,
                modified: ye
            }].concat(lcs(xb, yb, isEqual));
        }

        var a = lcs(xs, yb, isEqual);
        var b = lcs(xb, ys, isEqual);
        return a.length > b.length ? a : b;
    }

    return [];
}

/**
 * Diff two lists of items.
 * @param  {List}  original
 * @param  {List}  modified
 * @param  {Function} isVariant(a, b): Return true if a and b are modifiable items
 * @param  {Function} isEqual(a, b): Return true if a and b are stricly equal
 * @return {List<Change>}
 */
function diff(original, modified, isVariant, isEqual) {
    original = original.toArray();
    modified = modified.toArray();
    var subsequence = lcs(original, modified, isVariant);

    var result = [];
    var sfirst = void 0,
        mfirst = void 0,
        ofirst = void 0;

    while (subsequence.length > 0) {
        var _subsequence = subsequence;

        var _subsequence2 = _toArray(_subsequence);

        sfirst = _subsequence2[0];
        subsequence = _subsequence2.slice(1);


        while (modified.length > 0) {
            var _modified = modified;

            var _modified2 = _toArray(_modified);

            mfirst = _modified2[0];
            modified = _modified2.slice(1);

            if (isEqual(mfirst, sfirst.modified)) {
                break;
            }

            result.push(Change.create({
                type: TYPES.ADDED,
                modified: mfirst
            }));
        }

        while (original.length > 0) {
            var _original = original;

            var _original2 = _toArray(_original);

            ofirst = _original2[0];
            original = _original2.slice(1);

            if (isEqual(ofirst, sfirst.original)) {
                break;
            }

            result.push(Change.create({
                type: TYPES.REMOVED,
                original: ofirst
            }));
        }

        result.push(Change.create({
            type: isEqual(sfirst.original, sfirst.modified) ? TYPES.IDENTICAL : TYPES.MODIFIED,
            original: sfirst.original,
            modified: sfirst.modified
        }));
    }

    while (modified.length > 0) {
        var _modified3 = modified;

        var _modified4 = _toArray(_modified3);

        mfirst = _modified4[0];
        modified = _modified4.slice(1);

        result.push(Change.create({
            type: TYPES.ADDED,
            modified: mfirst
        }));
    }

    while (original.length > 0) {
        var _original3 = original;

        var _original4 = _toArray(_original3);

        ofirst = _original4[0];
        original = _original4.slice(1);


        result.push(Change.create({
            type: TYPES.REMOVED,
            original: ofirst
        }));
    }

    return List(result);
}

module.exports = diff;
module.exports.TYPES = TYPES;